package core.api.controller;

import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatDayInfo;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatWeekInfo;
import core.services.ServiceExitCode;
import core.services.ServiceResult;
import core.services.TaskSeriesService;
import core.services.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class TaskSeriesController {

    @Autowired private TaskService taskService;
    @Autowired private TaskSeriesService taskSeriesService;

    @PostMapping("/tasks/{taskID}/taskseries/weekly")
    public ResponseEntity<?> onCreateTaskSeries(@PathVariable long taskID, @Valid @RequestBody TaskRepeatWeekInfo taskRepeatWeekInfo) {
        var taskPermission = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(taskPermission.hasIssue()) {
            return taskPermission.mapToResponseEntity();
        } else {
            ServiceExitCode serviceExitCode = taskSeriesService.createTaskSeries(taskPermission.getResult(), taskRepeatWeekInfo);
            return serviceExitCode.mapToResponseEntity();
        }
    }

    @PostMapping("/tasks/{taskID}/taskseries/daily")
    public ResponseEntity<?> onCreateTaskSeries(@PathVariable long taskID, @Valid @RequestBody TaskRepeatDayInfo taskRepeatDayInfo) {
        var taskPermission = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(taskPermission.hasIssue()) {
            return taskPermission.mapToResponseEntity();
        } else {
            ServiceExitCode serviceExitCode = taskSeriesService.createTaskSeries(taskPermission.getResult(), taskRepeatDayInfo);
            return serviceExitCode.mapToResponseEntity();
        }
    }
}
