package core.services;

import core.api.models.auth.SimpleStatusResponse;
import org.springframework.http.ResponseEntity;

public class PermissionResult <T> extends ServiceResult<T> {

    private boolean hasPermissions;
    public PermissionResult(ServiceExitCode exitCode, T result) {
        super(exitCode, result);
    }

    public PermissionResult(T result) {
        super(result);
    }

    public PermissionResult(ServiceExitCode exitCode) {
        super(exitCode);
    }

    public PermissionResult(ServiceExitCode exitCode, T result, boolean hasPermissions) {
        super(exitCode, result);
        this.hasPermissions = hasPermissions;
    }

    public PermissionResult(T result, boolean hasPermissions) {
        super(result);
        this.hasPermissions = hasPermissions;
    }

    public PermissionResult(ServiceExitCode exitCode, boolean hasPermissions) {
        super(exitCode);
        this.hasPermissions = hasPermissions;
    }

    public boolean isNoPermissions() {
        return !hasPermissions;
    }

    public void setHasPermissions(boolean hasPermissions) {
        this.hasPermissions = hasPermissions;
    }

    @Override
    public ResponseEntity<?> mapToResponseEntity() {
        if(isNoPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        } else {
            return super.mapToResponseEntity();
        }
    }

    @Override
    public boolean hasIssue() {
        return super.hasIssue() || isNoPermissions();
    }
}
