package core.services;

import core.api.models.timemanager.tasks.repeatinginfo.DeadlineStrategy;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatDayInfo;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatWeekInfo;
import core.entities.timemanager.Task;
import core.entities.timemanager.TaskSerie;
import core.entities.timemanager.TaskSerieItem;
import core.repositories.timemanager.TaskRepository;
import core.repositories.timemanager.TaskSerieItemRepository;
import core.repositories.timemanager.TaskSeriesRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

@Service
public class TaskSeriesService {

    @Autowired private TaskRepository taskRepository;
    @Autowired private TaskSeriesRepository taskSeriesRepository;
    @Autowired private TaskSerieItemRepository taskSerieItemRepository;



    public ServiceExitCode createTaskSeries(Task rootTask, TaskRepeatWeekInfo taskRepeatInfo) {
        return ServiceExitCode.OK;
    }

    public ServiceExitCode createTaskSeries(Task rootTask, TaskRepeatDayInfo taskRepeatInfo) {
        if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.FIX_DEADLINE) {
            return ServiceExitCode.INVALID_PARAMETER;
        }

        List<Task> taskList = new ArrayList<>();
        taskList.add(rootTask);
        LocalDate currentTaskDate = rootTask.getStartDate().plusDays(taskRepeatInfo.getOffset());
        TaskSerie taskSerie = new TaskSerie();

        TaskSerieItem rootItem = taskSerie.addTask(rootTask);
        rootTask.setTaskSerieItem(rootItem);
        int index = 2;
        while(currentTaskDate.isBefore(taskRepeatInfo.getEndingDate())) {
            Task task = Task.cloneTask(rootTask);
            task.setStartDate(currentTaskDate);
            if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_EQUAL_START) {
                task.setDeadline(currentTaskDate);
            } else if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_FIT_START) {
                task.setDeadline(currentTaskDate.plusDays(taskRepeatInfo.getOffset()-1));
            }
            TaskSerieItem taskSerieItem = taskSerie.addTask(task);
            taskList.add(task);
            task.setTaskSerieItem(taskSerieItem);
            currentTaskDate = currentTaskDate.plusDays(taskRepeatInfo.getOffset());
            index++;
        }

        taskSeriesRepository.save(taskSerie);
        taskRepository.saveAll(taskList);
        taskSerieItemRepository.saveAll(taskSerie.getTasks());

        return ServiceExitCode.OK;
    }

    public void deleteTaskSeriesItem(Task task) {
        TaskSerieItem item = task.getTaskSerieItem();
        TaskSerie taskSerie = task.getTaskSerieItem().getTaskSerie();
        task.setTaskSerieItem(null);
        taskSerieItemRepository.delete(item);
        if(taskSerie.getTasks().size() == 1) {
            taskSeriesRepository.delete(taskSerie);
        }
    }

    public static String convertIndexToString(int index) {
        if(index < 10) {
            return "0" + index;
        } else {
            return String.valueOf(index);
        }
    }

    public static String insertNameIndex(int seriesIndex, String taskName) {
        return taskName.replaceAll("\\$\\{i}", convertIndexToString(seriesIndex));
    }
}
